# Copyright (c) 2003, James Seng. (http://james.seng.cc/)
# This code is released under the Artistic License.

package MT::App::BayesianTrain;
use strict;

use MT::Util qw( format_ts encode_html );

use MT::App::CMS;
@MT::App::BayesianTrain::ISA = qw( MT::App::CMS );

my %Limit_Multipliers = (
    Individual => 1,
    Daily => 2,
    Weekly => 5,
    Monthly => 10,
);

sub init {
    my $app = shift;
    $app->SUPER::init(@_) or return;
    $app->add_methods(
            # override these ( MT::App::CMS )
            'list_blogs' => \&list_blogs,
            'menu'      => \&show_menu,
            'rebuild'   => \&rebuild_pages,

            'list_comments' => \&list_comments,
            'train_comments' => \&train_comments,
            'delete_comment_spams' => \&delete_comment_spams,

            'list_pings' => \&list_pings,
            'train_pings' => \&train_pings,
            'delete_ping_spams' => \&delete_ping_spams,
    );
    $app->{default_mode} = 'list_blogs';
    $app->{template_dir} = 'cms';
    $app->{requires_login} = 1;
    $app->{user_class} = 'MT::Author';
    $app;
}

# Original from MT::App::CMS list_blogs
sub list_blogs {
    my $app = shift;
    my $q = $app->{query};
    require MT::Blog;
    require MT::Permission;
    require MT::Entry;
    require MT::Comment;
    my $author = $app->{author};
    my @perms = MT::Permission->load({ author_id => $author->id });
    my %perms = map { $_->blog_id => $_ } @perms;
    my @blogs = MT::Blog->load;
    my @data;
    my %param;
    my $i = 1;
    for my $blog (@blogs) {
        my $blog_id = $blog->id;
        my $perms = $perms{ $blog_id };
        next unless $perms && $perms->role_mask;
        $param{can_edit_authors} = 1 if $perms->can_edit_authors;
        my $row = { id => $blog->id, name => uc($blog->name),
                    description => $blog->description };
        $row->{is_odd} = $i++ % 2 ? 1 : 0;
        $row->{num_entries} = MT::Entry->count({ blog_id => $blog_id });
        $row->{num_comments} = MT::Comment->count({ blog_id => $blog_id });
        $row->{num_authors} = 0;
        my $iter = MT::Permission->load_iter({ blog_id => $blog_id });
        while (my $p = $iter->()) {
            $row->{num_authors}++ if $p->role_mask > 0;
        }
        $row->{can_delete} = $perms->can_edit_config;
        push @data, $row;
    }
    $param{blog_loop} = \@data;
    $param{can_create_blog} = $author->can_create_blog;
    $param{can_view_log} = $author->can_view_log;
    $param{saved_deleted} = $q->param('saved_deleted');
    $param{recalculated} = $q->param('recalculated');
    $param{author_name} = $author->name;
    $app->build_page('bayesian_list_blog.tmpl', \%param);
}

# Original from MT::App::CMS show_menu
sub show_menu {
    my $app = shift;
    my $perms = $app->{perms}
        or return $app->error($app->translate("No permissions"));
    require MT::Comment;
    require MT::TBPing;
    require MT::Trackback;
    require MT::Permission;
    require MT::Entry;
    my $blog_id = $app->{query}->param('blog_id');
    my $iter = MT::Entry->load_iter({ blog_id => $blog_id },
        { 'sort' => 'created_on',
          direction => 'descend',
          limit => 5 });
    my @e_data;
    my $i = 1;
    my $author_id = $app->{author}->id;
    while (my $entry = $iter->()) {
        my $row = { entry_id => $entry->id,
                    entry_blog_id => $entry->blog_id, };
        unless ($row->{entry_title} = $entry->title) {
            my $title = remove_html($entry->text);
            $row->{entry_title} = substr($title, 0, 22) . '...';
        }
        $row->{entry_title} = encode_html($row->{entry_title}, 1);
        $row->{entry_created_on} = format_ts("%Y.%m.%d", $entry->created_on);
        $row->{is_odd} = $i++ % 2;
        $row->{has_edit_access} = $perms->can_edit_all_posts ||
            $entry->author_id == $author_id;
        push @e_data, $row;
    }
    $iter = MT::Comment->load_iter({ blog_id => $blog_id },
        { 'sort' => 'created_on',
          direction => 'descend',
          limit => 5 });
    my @c_data;
    $i = 1;
    while (my $comment = $iter->()) {
        my $row = { comment_id => $comment->id,
                    comment_author => $comment->author || '[No author]',
                    comment_blog_id => $comment->blog_id, };
        $row->{comment_created_on} = format_ts("%Y.%m.%d",
            $comment->created_on);
        my $entry = MT::Entry->load($comment->entry_id);
        $row->{has_edit_access} = $perms->can_edit_all_posts ||
            $entry->author_id == $author_id;
        $row->{is_odd} = $i++ % 2;
        push @c_data, $row;
    }
    $iter = MT::TBPing->load_iter({ blog_id => $blog_id },
        { 'sort' => 'created_on',
          direction => 'descend',
          limit => 5 });
    my @p_data;
    $i = 1;
    while (my $ping = $iter->()) {
        my $row = { ping_id => $ping->id,
                    ping_title => $ping->title || '[No title]',
                    ping_url => $ping->source_url,
                    ping_blog_id => $ping->blog_id, };
        $row->{ping_created_on} = format_ts("%Y.%m.%d", $ping->created_on);
        #my $tb = MT::Trackback->load($ping->tb_id);
        #my $entry = MT::Entry->load($tb->entry_id);
        #$row->{has_edit_access} = $perms->can_edit_all_posts ||
        #    $entry->author_id == $author_id;
        #$row->{ping_entry_id} = $entry->id;
        $row->{is_odd} = $i++ % 2;
        push @p_data, $row;
    }
    require MT::Blog;
    my $blog = MT::Blog->load($blog_id);
    my %param = (entry_loop => \@e_data, comment_loop => \@c_data,
                 ping_loop => \@p_data);
    $param{blog_description} = $blog->description;
    $param{welcome} = $blog->welcome_msg;
    $param{num_entries} = MT::Entry->count({ blog_id => $blog_id });
    $param{num_comments} = MT::Comment->count({ blog_id => $blog_id });
    $param{num_authors} = 0;
    $iter = MT::Permission->load_iter({ blog_id => $blog_id });
    while (my $p = $iter->()) {
        $param{num_authors}++ if $p->role_mask > 0;
    }
    $app->build_page('bayesian_menu.tmpl', \%param);
}

# Original from MT::App::CMS rebuild_pages
sub rebuild_pages {
    my $app = shift;
    my $perms = $app->{perms}
        or return $app->error($app->translate("No permissions"));
    require MT::Entry;
    require MT::Blog;
    my $q = $app->{query};
    my $blog_id = $q->param('blog_id');
    my $blog = MT::Blog->load($blog_id);
    my $order = $q->param('type');
    my @order = split /,/, $order;
    my $next = $q->param('next');
    my $done = 0;
    my $type = $order[$next];
    $next++;
    $done++ if $next >= @order;
    my($offset, $limit);
    my $total_entries = $q->param('total_entries');

    ## Tells MT::_rebuild_entry_archive_type to cache loaded templates so
    ## that each template is only loaded once.
    $app->{cache_templates} = 1;

    my($tmpl_saved);
    if ($type eq 'all') {
        return $app->error($app->translate("Permission denied."))
            unless $perms->can_rebuild;
        $app->rebuild( BlogID => $blog_id )
            or return;
    } elsif ($type eq 'index') {
        return $app->error($app->translate("Permission denied."))
            unless $perms->can_rebuild;
        $app->rebuild_indexes( BlogID => $blog_id ) or return;
    } elsif ($type =~ /^index-(\d+)$/) {
        return $app->error($app->translate("Permission denied."))
            unless $perms->can_rebuild;
        my $tmpl_id = $1;
        require MT::Template;
        $tmpl_saved = MT::Template->load($tmpl_id);
        $app->rebuild_indexes( BlogID => $blog_id, Template => $tmpl_saved,
                               Force => 1 )
            or return;
        $order = "index template '" . $tmpl_saved->name . "'";
    } elsif ($type =~ /^entry-(\d+)$/) {
        my $entry_id = $1;

        # Insert spam recalculation here
        $app->_recalculate($blog_id,$entry_id);

        require MT::Entry;
        my $entry = MT::Entry->load($entry_id);
        return $app->error("Permission denied.")
            unless $perms->can_edit_entry($entry, $app->{author});
        $app->rebuild_entry( Entry => $entry, BuildDependencies => 1 )
            or return;
        $order = "entry '" . $entry->title . "'";
    } elsif ($Limit_Multipliers{$type}) {
        return $app->error($app->translate("Permission denied."))
            unless $perms->can_rebuild;
        $offset = $q->param('offset') || 0;
        $limit = $q->param('limit');
        if ($offset < $total_entries) {
            # Insert spam recalculation here
            for (my $i=$offset;$i<$offset+$limit && $i<$total_entries;$i++) {
                $app->_recalculate($blog_id,$i);
            }
            $app->rebuild( BlogID => $blog_id,
                           ArchiveType => $type,
                           NoIndexes => 1,
                           Offset => $offset,
                           Limit => $limit )
                or return;
            $offset += $limit;
        }
        if ($offset < $total_entries) {
            $done-- if $done;
            $next--;
        } else {
            $offset = 0;
        }
    } elsif ($type) {
        return $app->error($app->translate("Permission denied."))
            unless $perms->can_rebuild;
        $app->rebuild( BlogID => $blog_id,
                       ArchiveType => $type,
                       NoIndexes => 1 )
            or return;
    }
    unless ($done) {
        my $type_name = $order[$next];
        ## If we're moving on to the next rebuild step, recalculate the
        ## limit.
        if (defined($offset) && $offset == 0) {
            if (my $mult = $Limit_Multipliers{$type_name}) {
                $limit = $app->{cfg}->EntriesPerRebuild * $mult;
            }
        }
        my %param = ( build_type => $order, build_next => $next,
                      build_type_name => $type_name,
                      total_entries => $total_entries,
                      offset => $offset, limit => $limit,
                      is_bm => scalar $q->param('is_bm'),
                      entry_id => scalar $q->param('entry_id'),
                      is_new => scalar $q->param('is_new'),
                      old_status => scalar $q->param('old_status') );
        if ($Limit_Multipliers{$type_name}) {
            $param{is_individual} = 1;
            $param{indiv_range} = sprintf "%d - %d", $offset+1,
                $offset + $limit > $total_entries ? $total_entries :
                $offset + $limit;
        }
        $app->build_page('rebuilding.tmpl', \%param);
    } else {
        if ($q->param('entry_id')) {
            require MT::Entry;
            my $entry = MT::Entry->load(scalar $q->param('entry_id'));
            require MT::Blog;
            my $blog = MT::Blog->load($entry->blog_id);
            my $list = $app->needs_ping( Entry => $entry, Blog => $blog,
                OldStatus => scalar $q->param('old_status') );
            if ($entry->status == MT::Entry::RELEASE() && $list) {
                my @urls = map { { url => $_ } } @$list;
                $app->build_page('pinging.tmpl', { blog_id => $blog->id,
                    entry_id => $entry->id,
                    old_status => scalar $q->param('old_status'),
                    is_new => scalar $q->param('is_new'),
                    url_list => \@urls,
                    is_bm => scalar $q->param('is_bm') });
            } else {
                $app->_finish_rebuild_ping($entry, scalar $q->param('is_new'));
            }
        } else {
            my $all = $order =~ /,/;
            my $type = $order;
            my $is_one_index = $order =~ /index template/;
            my $is_entry = $order =~ /entry/;
            my %param = ( all => $all, type => $type,
                          is_one_index => $is_one_index,
                          is_entry => $is_entry );
            if ($is_one_index) {
                $param{tmpl_url} = $blog->site_url;
                $param{tmpl_url} .= '/' if $param{tmpl_url} !~ m!/$!;
                $param{tmpl_url} .= $tmpl_saved->outfile;
            }
            $app->build_page('rebuilt.tmpl', \%param);
        }
    }
}
sub list_comments {
    my $app = shift;
    my $q = $app->{query};
    my $blog_id = $q->param('blog_id');
    my $author = $app->{author};
    require MT::Permission;
    my $perms = MT::Permission->load({author_id => $author->id,
                                      blog_id => $blog_id });

    if (!$perms) {
        return $app->error($app->translate(
            "You are not authorized to log in to this blog."))
    }

    if (!$perms->can_edit_all_posts && !$perms->can_post) {
        return $app->error($app->translate("Permission denied."));
    }

    my %param;
    my %terms = (blog_id => $blog_id);
    my $limit = $q->param('limit') || 20;
    $param{"limit_" . $limit} = 1;
    my $offset = $q->param('offset') || 0;
    ## We load $limit + 1 records so that we can easily tell if we have a
    ## page of next entries to link to. Obviously we only display $limit
    ## entries.
    my %arg = (
            'sort' => 'created_on',
            direction => 'descend',
            ($limit eq 'none' ? () : (limit => $limit + 1)),
            ($offset ? (offset => $offset) : ()),
    );
    if (!$perms->can_edit_all_posts) {
        $terms{author_id} = $app->{author}->id;
    }

    my($iter);

    require MT::Comment;
    $iter = MT::Comment->load_iter({ blog_id => $blog_id }, \%arg);

    my @data;
    my $i = 1;
    while (my $obj = $iter->()) {
        my $row = $obj->column_values;

        $row->{created_on_formatted} = format_ts("%Y.%m.%d", $obj->created_on);
        $row->{created_on_time_formatted} =
            format_ts("%Y-%m-%d %H:%M:%S", $obj->created_on);

        $row->{author_info} = $obj->author;
        $row->{author_info} .= "&nbsp; &lt;" . $obj->email . "&gt;" 
            if ($obj->email);
        $row->{author_info} .= "<br/>" . 
            sprintf("<a href=\"%s\">%s</a>",$obj->url,$obj->url)
            if ($obj->url);
        $row->{author_info} .= "<br/>IP: " . $obj->ip
            if ($obj->ip);
        $row->{author_info} .= "<br/>Date: " . $obj->created_on_time_formatted;

        $row->{entry_odd} = $i++ %2 ? 1 : 0;

        require MT::Bayesian;
        my $by = MT::Bayesian->load({blog_id => $blog_id, 
                                  comment_id => $obj->id });
        if (!$by) {
            $by = MT::Bayesian->new;
            $by->new_bayesian( {blog_id => $blog_id,
                                comment_id => $obj->id} );
        } 

        $row->{prob} = int $by->prob*100;
        if ($by->spam>0) {
            $row->{spam} = "checked";
        } elsif ($by->spam<0 && $by->is_trained) {
            $row->{notspam} = "checked";
        }

        require MT::Entry;
        my $en = MT::Entry->load($obj->entry_id);
        $obj->text(sprintf("<b>%s</b><p>%s",
            $en->title,$obj->text)) if ($en);

        $obj->text($obj->text . 
            sprintf("<p>[Spam Probability: %d%%]",$row->{prob}));

        push @data, $row;
    }

    if ($limit ne 'none') {
        my $have_next_entry = scalar @data == $limit + 1;
        pop @data if $have_next_entry;
        if ($offset) {
            $param{prev_offset} = 1;
            $param{prev_offset_val} = $offset - $limit;
        }
        if ($have_next_entry) {
            $param{next_offset} = 1;
            $param{next_offset_val} = $offset + $limit;
        }
    }
    $param{object_loop} = \@data;
    $param{blog_id} = $blog_id;
    $param{can_rebuild} = 1;
    $param{saved_deleted} = $q->param('saved_deleted');
    $param{recalculated} = $q->param('recalculated');
    $param{saved} = $q->param('saved');
    $param{limit} = $limit;
    $param{offset} = $offset;
    $param{author_name} = $author->name;
    $app->build_page("bayesian_list_comments.tmpl", \%param);
}

sub train_comments {
    my $app = shift;
    my $q = $app->{query};
    my $blog_id = $q->param('blog_id');
    my $author = $app->{author};
    require MT::Permission;
    my $perms = MT::Permission->load({author_id => $author->id,
                                      blog_id => $blog_id });
    if (!$perms) {
        return $app->error($app->translate(
            "You are not authorized to log in to this blog."))
    }

    if (!$perms->can_edit_all_posts && !$perms->can_post) {
        return $app->error($app->translate("Permission denied."));
    }

    my @nspam;
    require MT::Bayesian;
    for my $notspam ($q->param('n')) {
        push @nspam, $notspam;

        my $by = MT::Bayesian->load({blog_id => $blog_id, 
                                     comment_id => $notspam});
        if (!$by) {
            $by = MT::Bayesian->new;
            $by->new_bayesian( {blog_id => $blog_id,
                                comment_id => $notspam} );
        }
        $by->train_nonspam;
    }

    for my $spam ($q->param('s')) {
        my $repeat = 0;
        foreach my $ns (@nspam) {
            $repeat = 1 if ($ns eq $spam);
        }
        next if $repeat;

        my $by = MT::Bayesian->load({blog_id => $blog_id, 
                                     comment_id => $spam});
        if (!$by) {
            $by = MT::Bayesian->new;
            $by->new_bayesian( {blog_id => $blog_id,
                                comment_id => $spam} );
        }
        $by->train_spam;
    }

    my $url = $app->uri . '?__mode=list_comments&blog_id=' . $blog_id;
    if (my $o = $q->param('offset')) {
        $url .= '&offset=' . $o;
    }
    if (my $l = $q->param('limit')) {
        $url .= '&limit=' . $l;
    }
    $app->redirect($url . '&saved=1');
}

sub delete_comment_spams {
    my $app = shift;
    my $q = $app->{query};
    my $blog_id = $q->param('blog_id');
    my $allspam = $q->param('allspam');
    my $author = $app->{author};
    require MT::Permission;
    my $perms = MT::Permission->load({author_id => $author->id,
                                      blog_id => $blog_id });
    if (!$perms) {
        return $app->error($app->translate(
            "You are not authorized to log in to this blog."))
    }

    if (!$perms->can_edit_all_posts && !$perms->can_post) {
        return $app->error($app->translate("Permission denied."));
    }

    require MT::Bayesian;
    my @bys = MT::Bayesian->load({ blog_id => $blog_id });

    for my $obj (@bys) {
        next unless ($obj->comment_id);
        next unless ($allspam || $obj->is_trained);
        if ($obj->spam>0) {
            require MT::Comment;
            my $cm = MT::Comment->load($obj->comment_id);
            $cm->remove;
            $obj->remove;
        }
    }

    my $url = $app->uri . '?__mode=list_comments&blog_id=' . $blog_id;
    if (my $o = $q->param('offset')) {
        $url .= '&offset=' . $o;
    }
    if (my $l = $q->param('limit')) {
        $url .= '&limit=' . $l;
    }
    $app->redirect($url . '&saved_deleted=1');
}

sub list_pings {
    my $app = shift;
    my $q = $app->{query};
    my $blog_id = $q->param('blog_id');
    my $author = $app->{author};
    require MT::Permission;
    my $perms = MT::Permission->load({author_id => $author->id,
                                      blog_id => $blog_id });

    if (!$perms) {
        return $app->error($app->translate(
            "You are not authorized to log in to this blog."))
    }

    if (!$perms->can_edit_all_posts && !$perms->can_post) {
        return $app->error($app->translate("Permission denied."));
    }

    my %param;
    my %terms = (blog_id => $blog_id);
    my $limit = $q->param('limit') || 20;
    $param{"limit_" . $limit} = 1;
    my $offset = $q->param('offset') || 0;
    ## We load $limit + 1 records so that we can easily tell if we have a
    ## page of next entries to link to. Obviously we only display $limit
    ## entries.
    my %arg = (
            'sort' => 'created_on',
            direction => 'descend',
            ($limit eq 'none' ? () : (limit => $limit + 1)),
            ($offset ? (offset => $offset) : ()),
    );
    if (!$perms->can_edit_all_posts) {
        $terms{author_id} = $app->{author}->id;
    }

    my($iter);

    require MT::TBPing;
    $iter = MT::TBPing->load_iter({ blog_id => $blog_id }, \%arg);

    my @data;
    my $i = 1;
    while (my $obj = $iter->()) {
        my $row = $obj->column_values;

        $row->{created_on_formatted} = format_ts("%Y.%m.%d", $obj->created_on);
        $row->{created_on_time_formatted} =
            format_ts("%Y-%m-%d %H:%M:%S", $obj->created_on);

        $row->{ping_info} = sprintf(
            "<a href=\"%s\">%s</a><br/>IP: %s<br/>Date: %s<br/>",
            $obj->source_url,$obj->blog_name,$obj->ip,
            $row->{created_on_time_formatted});

        $row->{entry_odd} = $i++ %2 ? 1 : 0;

        require MT::Bayesian;
        my $by = MT::Bayesian->load({blog_id => $blog_id, 
                                     ping_id => $obj->id });
        if (!$by) {
            $by = MT::Bayesian->new;
            $by->new_bayesian( {blog_id => $blog_id,
                                ping_id => $obj->id} );
        }

        $row->{prob} = int $by->prob*100;
        if ($by->spam>0) {
            $row->{spam} = "checked";
        } elsif ($by->spam<0 && $by->is_trained) {
            $row->{notspam} = "checked";
        }

        require MT::Trackback;
        my $tb = MT::Trackback->load($obj->tb_id);
        if ($tb) {
            require MT::Entry;
            my $en = MT::Entry->load($tb->entry_id);
            $obj->text(sprintf("<b>%s</b><p>", encode_html($en->title))) 
                if ($en);
        }
        $obj->text($obj->text.
            sprintf("<i>%s</i><br/>%s<p>[Spam Probability: %d%%]",
            $obj->title,encode_html($obj->excerpt),$row->{prob}));

        push @data, $row;
    }

    if ($limit ne 'none') {
        my $have_next_entry = scalar @data == $limit + 1;
        pop @data if $have_next_entry;
        if ($offset) {
            $param{prev_offset} = 1;
            $param{prev_offset_val} = $offset - $limit;
        }
        if ($have_next_entry) {
            $param{next_offset} = 1;
            $param{next_offset_val} = $offset + $limit;
        }
    }
    $param{object_loop} = \@data;
    $param{blog_id} = $blog_id;
    $param{can_rebuild} = 1;
    $param{recalculated} = $q->param('recalculated');
    $param{saved_deleted} = $q->param('saved_deleted');
    $param{saved} = $q->param('saved');
    $param{limit} = $limit;
    $param{offset} = $offset;
    $param{author_name} = $author->name;
    $param{recalculated} = $q->param('recalculated');
    $app->build_page("bayesian_list_pings.tmpl", \%param);
}

sub train_pings {
    my $app = shift;
    my $q = $app->{query};
    my $blog_id = $q->param('blog_id');
    my $author = $app->{author};
    require MT::Permission;
    my $perms = MT::Permission->load({author_id => $author->id,
                                      blog_id => $blog_id });
    if (!$perms) {
        return $app->error($app->translate(
            "You are not authorized to log in to this blog."))
    }

    if (!$perms->can_edit_all_posts && !$perms->can_post) {
        return $app->error($app->translate("Permission denied."));
    }

    my @nspam;
    require MT::Bayesian;
    for my $notspam ($q->param('n')) {
        push @nspam, $notspam;

        my $by = MT::Bayesian->load({blog_id => $blog_id, 
                                     ping_id => $notspam});
        if (!$by) {
            $by = MT::Bayesian->new;
            $by->new_bayesian( {blog_id => $blog_id,
                                ping_id => $notspam} );
        }
        $by->train_nonspam;
    }

    for my $spam ($q->param('s')) {
        my $repeat = 0;
        foreach my $ns (@nspam) {
            $repeat = 1 if ($ns eq $spam);
        }
        next if $repeat;

        my $by = MT::Bayesian->load({blog_id => $blog_id, 
                                     ping_id => $spam});
        if (!$by) {
            $by = MT::Bayesian->new;
            $by->new_bayesian( {blog_id => $blog_id,
                                ping_id => $spam} );
        }
        $by->train_spam;
    }

    my $url = $app->uri . '?__mode=list_pings&blog_id=' . $blog_id;
    if (my $o = $q->param('offset')) {
        $url .= '&offset=' . $o;
    }
    if (my $l = $q->param('limit')) {
        $url .= '&limit=' . $l;
    }
    $app->redirect($url . '&saved=1');
}

sub delete_ping_spams {
    my $app = shift;
    my $q = $app->{query};
    my $blog_id = $q->param('blog_id');
    my $allspam = $q->param('allspam');
    my $author = $app->{author};
    require MT::Permission;
    my $perms = MT::Permission->load({author_id => $author->id,
                                      blog_id => $blog_id });
    if (!$perms) {
        return $app->error($app->translate(
            "You are not authorized to log in to this blog."))
    }

    if (!$perms->can_edit_all_posts && !$perms->can_post) {
        return $app->error($app->translate("Permission denied."));
    }

    require MT::Bayesian;
    my @bys = MT::Bayesian->load({ blog_id => $blog_id });

    for my $obj (@bys) {
        next unless ($obj->ping_id);
        next unless ($allspam || $obj->is_trained);
        if ($obj->spam>0) {
            require MT::TBPing;
            my $ping = MT::TBPing->load($obj->ping_id);
            $ping->remove;
            $obj->remove;
        }
    }

    my $url = $app->uri . '?__mode=list_comments&blog_id=' . $blog_id;
    if (my $o = $q->param('offset')) { $url .= '&offset=' . $o;
    }
    if (my $l = $q->param('limit')) {
        $url .= '&limit=' . $l;
    }
    $app->redirect($url . '&saved_deleted=1');
}

sub _recalculate {
    require MT::Bayesian;

    my ($app,$blog_id,$entry_id) = @_;

    require MT::Comment;
    my @comments = MT::Comment->load({ blog_id => $blog_id, 
                                   entry_id => $entry_id });


    foreach my $cm (@comments) {
        my $by;

        my @bys = MT::Bayesian->load( { blog_id => $blog_id,
                                        comment_id => $cm->id });

        # clean up stray comments
        undef $by;
        foreach my $tby (@bys) {
            if (!$by) {
                $by = $tby;
            } else {
                $tby->remove;
            }
        }

        if (!$by) {
            $by = MT::Bayesian->new;
            $by->new_bayesian( { blog_id => $blog_id,
                                 comment_id => $cm->id } );
        } else {
            $by->test_spam(1);
        }
    }

    require MT::TBPing;
    my @pings = MT::TBPing->load( { blog_id => $blog_id, 
                                    entry_id => $entry_id });

    foreach my $tb (@pings) {
        my $by;

        my @pgs = MT::Bayesian->load( { blog_id => $blog_id,
                                        ping_id => $tb->id });

        # clean up stray comments
        undef $by;
        foreach my $tpg (@pgs) {
            if (!$by) {
                $by = $tpg;
            } else {
                $tpg->remove;
            }
        }

        if (!$by) {
            $by = MT::Bayesian->new;
            $by->new_bayesian( { blog_id => $blog_id,
                                 ping_id => $tb->id } );
        } else {
            $by->test_spam(1);
        }
    }
}

1;
