#!/usr/bin/perl -w

use strict;

my $MT_DIR;
BEGIN {
    if ($0 =~ m!(.*[/\\])!) {
        $MT_DIR = $1;
    } else {
        $MT_DIR = './';
    }
    unshift @INC, $MT_DIR . 'lib';
    unshift @INC, $MT_DIR . 'extlib';
}

use MT;
use MT::Bayesian;
use CGI;

local $| = 1;

print "Content-Type: text/html\n\n";
print "<pre>\n\n";

my $mt = MT->new(Config => $MT_DIR . 'mt.cfg',
                Directory => $MT_DIR) || die MT->errstr;

my $q = new CGI;
my $blog_id = $q->param('blog_id');
my $check_token = $q->param('check_token');

if (!$blog_id) {
    printf "Please select a blog to clean the database\n\n";

    use MT::Blog;
    my @blogs = MT::Blog->load;

    foreach my $b (@blogs) {
        printf "<a href=%s?blog_id=%d>%s</a><br>",
            $q->url,$b->id,$b->name;
    }

    printf "</pre>";

    exit;
}

my @cids = {};
my @pids = {};
my @bys = MT::Bayesian->load( { blog_id => $blog_id });
for my $obj (@bys) {
    if ($obj->comment_id) {
        my ($error, $c);
        $error = 0;
        for my $c (@cids) {
            $error = 1 if ($obj->comment_id eq $c);
        }
    
        if ($error) {
            printf "comment: %d -- repeated -- not ok\n", $obj->comment_id;
            $obj->remove;
            next;
        }
        push @cids, $obj->comment_id;

        #$obj->test_spam(1);
        printf "comment: %d -- %d -- ok\n", $obj->comment_id, $obj->prob*100;
    } else {
        my ($error, $c);
        $error = 0;
        for my $c (@pids) {
            $error = 1 if ($obj->ping_id eq $c);
        }
    
        if ($error) {
            printf "ping: %d -- repeated -- not ok\n", $obj->ping_id;
            $obj->remove;
            next;
        }
        push @pids, $obj->ping_id;

        #$obj->test_spam(1);
        printf "ping: %d -- %d -- ok\n", $obj->ping_id, $obj->prob*100;
    }
}

exit if (!$check_token || $check_token ne 'yes');

use MT::BayesianToken;
my @tokens = MT::BayesianToken->load( { blog_id => $blog_id });

my @spam = {};
my @nonspam = {};
for my $tk (@tokens) {
    if ($tk->spam>0) {
        my $error = 0;
        for my $c (@spam) {
            $error = 1 if ($c eq $tk->token);
        }

        if ($error) {
            printf "!!! token: %s (%d) - repeated\n", $tk->token, $tk->spam;
            $tk->remove;
            next;
        }

        push @spam, $tk->token;
    } elsif ($tk->spam<=0) {
        my $error = 0;
        for my $c (@nonspam) {
            $error = 1 if ($c eq $tk->token);
        }

        if ($error) {
            printf "!!! token: %s (%d) - repeated\n", $tk->token, $tk->spam;
            $tk->remove;
            next;
        }

        push @nonspam, $tk->token;
    }
    printf "token: %s (%d) - %d\n", $tk->token, $tk->spam, $tk->token_count;
}
