# Copyright (c) 2003, James Seng. (http://james.seng.cc/)
# This code is released under the Artistic License.

use MT::Template::Context;
require MT::Bayesian;

MT::Template::Context->add_tag(SpamProb => sub {
    my ($ctx,$args) = @_;
    my $pg  = $ctx->stash('ping');
    my $cm = $ctx->stash('comment');
    my $blog_id = $ctx->stash('blog_id');

    return "?"  unless ($blog_id);

    if ($cm) {
        my $by = MT::Bayesian->load({blog_id => $blog_id, comment_id => $cm->id });
        if (!$by) {
            $by = MT::Bayesian->new;
            $by->new_bayesian( {blog_id => $blog_id, comment_id => $cm->id} );
        }
        return (int $by->prob*100);
    } elsif ($pg) {
        my $by = MT::Bayesian->load({blog_id => $blog_id, ping_id => $pg->id });
        if (!$by) {
            $by = MT::Bayesian->new;
            $by->new_bayesian( {blog_id => $blog_id, ping_id => $pg->id} );
        }
        return (int $by->prob*100);
    }

    return "?";
});

MT::Template::Context->add_conditional_tag(IfSpam => sub {
    my ($ctx,$args) = @_;
    my $pg  = $ctx->stash('ping');
    my $cm = $ctx->stash('comment');
    my $blog_id = $ctx->stash('blog_id');

    return 0 unless ($blog_id);

    if ($cm) {
        my $by = MT::Bayesian->load({blog_id => $blog_id, comment_id => $cm->id });
        if (!$by) {
            $by = MT::Bayesian->new;
            $by->new_bayesian( {blog_id => $blog_id, comment_id => $cm->id} );
        }
        return ($by->spam>0);
    } elsif ($pg) {
        my $by = MT::Bayesian->load({blog_id => $blog_id, ping_id => $pg->id });
        if (!$by) {
            $by = MT::Bayesian->new;
            $by->new_bayesian( {blog_id => $blog_id, ping_id => $pg->id} );
        }
        return ($by->spam>0);
    }

    return 0;
});

MT::Template::Context->add_conditional_tag(IfNotSpam => sub {
    my ($ctx,$args) = @_;
    my $pg  = $ctx->stash('ping');
    my $cm = $ctx->stash('comment');
    my $blog_id = $ctx->stash('blog_id');

    return 1 unless ($blog_id);

    if ($cm) {
        my $by = MT::Bayesian->load({blog_id => $blog_id, comment_id => $cm->id });
        if (!$by) {
            $by = MT::Bayesian->new;
            $by->new_bayesian( {blog_id => $blog_id, comment_id => $cm->id} );
        }
        return ($by->spam<1);
    } elsif ($pg) {
        my $by = MT::Bayesian->load({blog_id => $blog_id, ping_id => $pg->id });
        if (!$by) {
            $by = MT::Bayesian->new;
            $by->new_bayesian( {blog_id => $blog_id, ping_id => $pg->id} );
        }
        return ($by->spam<1);
    }

    return 1;
});

1;
